const {fork} = require('child_process')
const path = require('path')
const EventEmitter = require('events')

let queryProcess = null
let overclockProcess = null
let boostProcess = null

module.exports = new EventEmitter()

/*This starts an instance of GPU info for either query,overclock, or boost*/
function run(params, proc, processName) {
    if (proc) {
        module.exports.emit('driver-error', processName + ' already exists')
        return
    }

    env = process.env
    env.LD_LIBRARY_PATH = path.resolve(__dirname, '..', 'runtime').replace('app.asar', 'app.asar.unpacked')

    proc = fork(path.resolve(__dirname, 'child.js'), params, {
        silent: true,
        env:env,
    })

    proc.stdout.on('data', data => {
            try {
                const dataObj = JSON.parse(data.toString())
                module.exports.emit(dataObj.type + "-results", JSON.stringify(dataObj.payload))
            } catch (e) {
                module.exports.emit('misc-results', data.toString())
            }
        }
    )

    proc.on('close', code => {
        if (code !== 0)
            module.exports.emit('driver-error', processName + ' crashed')
        proc = null
    })
}

module.exports.query = (params) => {
    run(params, queryProcess, 'query')
}

module.exports.setOverclockSettings = (params) => {
    run(params, overclockProcess, 'overclock')
}

module.exports.startBoost = () => {
    const params = [JSON.stringify({boost:true, nvidia:true})]

    if (boostProcess) {
        module.exports.emit('driver-error', 'boost already exists')
        return
    }

    boostProcess = fork(path.resolve(__dirname, 'child.js'), params, {
        silent: true
    })

    boostProcess.stdout.on('data', data => {
            try {
                const dataObj = JSON.parse(data.toString())
                module.exports.emit(dataObj.type + '-results', JSON.stringify(dataObj.payload))
            } catch (e) {
                module.exports.emit('misc-results', data.toString())
            }
        }
    )

    boostProcess.on('close', code => {
        if (code !== 0)
            module.exports.emit('driver-error', 'boost crashed')
        boostProcess = null
    })

}

module.exports.stopBoost = () => {
    if (boostProcess) {
        boostProcess.kill()
        boostProcess  = null
    }
}

process.on('exit', () => {
    if (queryProcess) {
        queryProcess.kill()
    }
    if (overclockProcess) {
        overclockProcess.kill()
    }
    if (boostProcess) {
        boostProcess.kill()
    }
})
